import NextGenThemeProvider, { importThemeFonts } from '../NextGenThemeProvider';
import Button from '@mui/material/Button';
import { createTheme } from '@mui/material/styles';
import { render } from '@testing-library/react';

const defaultFontParams = {
    'en-US': {
        defaultFont: '',
        fontsToLoad: [],
        typographyVariants: { 'body-s': {} },
        section: {'body': ''}
    }
};

describe('NuskinThemeProvider', () => {
    const theme = createTheme({
        palette: {
            primary: {
                main: '#f00',
            },
        },
    });
    const commonText = 'Nu Skin App';
    it('1. renders without crashing', () => {
        const { getByText } = render(
            <NextGenThemeProvider theme={theme}>
                <div>{commonText}</div>
            </NextGenThemeProvider>,
        );
        expect(getByText(commonText)).toBeInTheDocument();
    });

    it('2. applies the correct theme to ThemeProvider', () => {
        const { getByText } = render(
            <NextGenThemeProvider theme={theme}>
                <Button variant="contained">{commonText}</Button>
            </NextGenThemeProvider>,
        );
        const comp = getByText(commonText);
        expect(comp).toHaveStyle('background-color: rgb(178, 0, 0)');
    });

    it('3. applies the correct theme to ThemeProvider with SSR', () => {
        const { getByText } = render(
            <NextGenThemeProvider theme={theme} isSSR>
                <Button variant="contained">{commonText}</Button>
            </NextGenThemeProvider>,
        );
        const comp = getByText(commonText);
        expect(comp).toHaveStyle('background-color: rgb(178, 0, 0)');
    });

    it('3. should not apply dark class when mode is not passed to NextGenThemeProvider', () => {
        const { container } = render(
            <NextGenThemeProvider theme={theme} isSSR>
                <Button variant="contained">{commonText}</Button>
            </NextGenThemeProvider>,
        );
        const comp = container.getElementsByClassName('dark');
        expect(comp).toHaveLength(0);
        expect(container.getElementsByClassName('light')).toHaveLength(1);
    });

    it('4. should apply dark class when mode is not passed to NextGenThemeProvider', () => {
        const { container } = render(
            <NextGenThemeProvider theme={theme} isSSR mode="dark">
                <Button variant="contained">{commonText}</Button>
            </NextGenThemeProvider>,
        );
        const comp = container.getElementsByClassName('dark');
        expect(comp).toHaveLength(1);
    });


});

describe('importThemeFonts utility function', () => {
    it('1. returns empty string if no fonts are provided', () => {
        const fonts = importThemeFonts(defaultFontParams as any);
        expect(fonts).toBe('');
    });

    it('2. returns empty string if no google fonts are provided', () => {
        const fonts = importThemeFonts({
            'en-US': {
                fontsToLoad: [
                    {
                        type: 'LocalFont',
                        fontName: 'Roboto',
                    }
                ],
            },
        } as any);
        expect(fonts).toBe('');
    });

    it('3. returns correct string if google fonts are provided', () => {
        const fonts = importThemeFonts({
            'en-US': {
                fontsToLoad: [
                    {
                        type: 'GoogleFont',
                        fontName: 'Roboto',
                        fontStyle: [400,  600],
                    },
                    {
                        type: 'GoogleFont',
                        fontName: 'Open Sans',
                        fontStyle: [400, 600],
                    },
                ],
            }
        } as any);
        const googleFontFamily = 'Roboto:wght@400;600&family=Open Sans:wght@400;600&';
        expect(fonts).toBe(
            `@import url('https://fonts.googleapis.com/css2?family=${googleFontFamily}display=swap');`
        );
    });

    it('4. returns correct string if google fonts are provided', () => {
        const fonts = importThemeFonts({
            'en-US': {
                fontsToLoad: [
                    {
                        type: 'GoogleFont',
                        fontName: 'Roboto',
                        fontStyle: [400,  600],
                    },
                    {
                        type: 'GoogleFont',
                        fontName: 'Open Sans',
                        fontStyle: [
                            [400, 0],
                            [600, 1]
                        ],
                    },
                ],
            }
        } as any);
        const googleFontFamily = 'Roboto:wght@400;600&family=Open Sans:ital,wght@0,400;0,600;1,600&';
        expect(fonts).toBe(
            `@import url('https://fonts.googleapis.com/css2?family=${googleFontFamily}display=swap');`
        );
    });

});
